/*
 * Test dataspace operations.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <ipc/mem_ipc.h>



int main(void)
{
  l4_cap_idx_t server = l4re_env_get_cap("server");

  /* Some memory to be mapped. */

  char *memory;
  unsigned long size = 40960;
  long err = ipc_attach_dataspace(server, size, (void **) &memory);

  if (err)
  {
    printf("Could not map memory: %s\n", l4sys_errtostr(err));
    return 1;
  }

  printf("Mapped memory at %p\n", memory);

  for (unsigned long offset = 0; offset < size; offset += 1024)
  {
    printf("10 bytes from %p...\n", (memory + offset));
    fwrite((memory + offset), sizeof(char), 10, stdout);
    fputs("\n", stdout);
  }

  return 0;
}
